#ifndef AbstractDynamicLibrary_hpp
#define AbstractDynamicLibrary_hpp

#include "DanuConfig.h"

namespace Danu
{
	class DANUIMPEXP AbstractDynamicLibrary
	{
		public:
			static void loadLibrary();
			static void unloadLibrary();
			static AbstractDynamicLibrary* getLibrary();
			static void	setLibrary(AbstractDynamicLibrary* aAbstractDynamicLibrary);

			virtual void load() 	= 0;
			virtual void unload() 	= 0;

		protected:
			AbstractDynamicLibrary();
			virtual ~AbstractDynamicLibrary();

		private:
			static AbstractDynamicLibrary* theDynamicLibrary;
	};

	/// Returns the DynamicLibrary object that represents the actual library
	/**
	 * If the user does not define USE_USER_DYNAMIC_LIBRARY in either DanuConfig.h
	 * or on the compilers commandline using -DUSE_USER_DYNAMIC_LIBRARY (GCC)
	 * DanuBase will supply it's own implementation of this function (see
	 * CreateDynamicLibrary.cpp and CreateDynamicLibrary.inl)
	 */
	AbstractDynamicLibrary* CreateDynamicLibrary();

}; // namespace Danu

#endif // AbstractDynamicLibrary_hpp
