#ifndef AbstractLockableObject_hpp
#define AbstractLockableObject_hpp

#include "DanuConfig.h"
#include <noncopyable.hpp>

namespace Danu
{
	/// The abstract base class for objects that can be locked.
	/**
	If a call in a thread to lock() succeeds the calling thread is said to be the owner of the object. As long as
	the owning thread does not call unlock() it will remain the owner of the object. If an object is owned, subsequent calls
	of lock() by other threads will block the calling thread untill the current owner releases the lock by calling unlock().
	*/
	class DANUIMPEXP AbstractLockableObject : public boost::noncopyable
	{
		public:
			virtual ~AbstractLockableObject(){}
			/**
			@param aTimeOut: if this is -1, this function should wait forever
			\todo Implement recursive locks or adjust docs
			*/
			virtual void                              lock( long aTimeOut = -1) = 0;
			virtual void                              trylock() = 0;
			virtual void                              unlock() = 0;
	};

	class DANUIMPEXP LOLock
	{
		public:
			LOLock( AbstractLockableObject* aAbstractLockableObject)
				:	lockableObject(aAbstractLockableObject)
			{
				lockableObject->lock();
			}
			~LOLock()
			{
				lockableObject->unlock();
			}
		private:
			AbstractLockableObject* lockableObject;
	};
}; // namespace Danu

#endif // AbstractLockableObject_hpp
