#ifndef DanuConfig_h
#define DanuConfig_h

// Copyright (c) 2003 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to Danu@Askesis.nl

/*! \file
	\brief The main Danu configuration file

	This file is a handcoded configure.h. This file contains the global MACROS that are needed for the difference in
	compiler, debug setting etc.. It should always be the first file included in a sourcefile, even before any compiler
	or platform header.

	This file includes the compiler dependant files by checking the compiler defined macros.

	The general idea is to set the global macros to a sensible value first and than to include the compiler specific
	to allow for compiler specific version of the macros.

	A sensible default is a macro that will allways go wrong.

	Macros should always be UPPERCASE, using "_" as word boundary if needed for readability

	Do not redefine ANSI macros, but redefine all compiler related macros to some DANU_xxxx macro so that in the code
	no compiler stuff is included other than throught the configuration files.

	$Date: 2003/12/24 08:07:56 $
	$Author: jkr $
	$Revision: 1.5 $

*/


/** \defgroup global_configuration_macros Global configuration macros */

/*@{*/

#define DANU_MAJOR   0
#define DANU_MINOR   1

// Implementation comment: because we do not generate platform dependant documentation we must create these defines
// globally as there is no WIN32 or linux defined in the DoxyGen preprocessor....
//! \todo Shouldn't this be done in the compiler section instead????
#define		DANU_PLATFORM_WINDOWS
//! \todo Shouldn't this be done in the compiler section instead????
#define		DANU_PLATFORM_LINUX

#if defined (_WIN32) || defined (__CYGWIN__)
	#undef DANU_PLATFORM_WINDOWS
	#define		DANU_PLATFORM_WINDOWS		1
	#undef		DANU_PLATFORM_LINUX
#elif linux
	#undef		DANU_PLATFORM_WINDOWS
	#undef		DANU_PLATFORM_LINUX
	#define		DANU_PLATFORM_LINUX			1
#else
	#error		You must define a platform!!!!
#endif

// Compiler string
#define DANU_COMPILER 					"unknown compiler"

#ifdef PRODUCTION
 	#define ALIB_TRACE_RUNTIME 1
	#define ALIB_DEBUG_RUNTIME 1
#elif DEVELOP
	#define ALIB_TRACE_ALL 1
	#define ALIB_DEBUG_ALL 1
#else
	#error		You must define a PRODUCTION or DEVELOP!!!!
#endif // ADEVELOP


// By default we use the library defined DefaultDynamicLibrary
// If we want to use our own implementation we must define
// at the compiler commandline (throught the Makefile)
#define USE_DEFAULT_DYNAMIC_LIBRARY			1
#ifdef USE_USER_DYNAMIC_LIBRARY
	#undef USE_DEFAULT_DYNAMIC_LIBRARY
#endif

// Not all compilers actualy have these macros:
// VC 6.0 has no __FUNCTION__ or __PRETTY_FUNCTION__
// GCC has does not demange __FUNCTION__ but __PRETTY_FUNCTION__ does etc.
// \todo: implement this in the compiler includes
#define __DANUFILE__			__FILE__
#define __DANUFUNCTION__		"Unknown function"
#define __DANULINE__			__LINE__

/*@}*/

// Compiler specific includes.
// Include these files as last so that they can reference the global stuff !!!
#ifdef __CYGWIN32__
#include "DanuCygwinConfig.h"
#endif // __CYGWIN32__

#ifdef __MINGW32__
#include "DanuMinGWConfig.h"
#endif // __MINGW32__

#ifdef linux
#include "DanuLinuxConfig.h"
#endif // linux

#ifdef _MSC_VER
#include "DanuVCConfig.h"
#endif // _MSC_VER

#endif // DanuConfig_h

