#include "DanuString.hpp"
#include <cstdarg>	// for: va_list va_start va_end vsprintf
					// They do not seem to be in the namespace std: ?

namespace Danu
{
	void stripLeadingSpaces(std::string& aString)
	{
		// No spaces in a zero length string...
		if(aString.length() == 0) return;

		const char* startOfRawString  = aString.data();
		const char* endOfRawString	 = startOfRawString + aString.length();

		unsigned long startPos		 = 0;

		// Strip the leading spaces of string
		while(isspace(*(startOfRawString+startPos)))
		{
			++startPos;
		}

		// Build the string
		const char* startOfStrippedString   = startOfRawString + startPos;
		const char* endOfStrippedString	   = endOfRawString;
		aString = std::string(startOfStrippedString,(endOfStrippedString - startOfStrippedString));
	}

	void stripTrailingSpaces(std::string& aString)
	{
		// No spaces in a zero length string...
		if(aString.length() == 0) return;

		const char* startOfRawString  = aString.data();
		const char* endOfRawString	 = startOfRawString + aString.length();

		unsigned long endPos 		 = 0;

		// Strip the trailing spaces of string
		while(isspace(*(endOfRawString-endPos-1)))
		{
			++endPos;
		}

		// Build the string
		const char* startOfStrippedString   = startOfRawString;
		const char* endOfStrippedString	   = endOfRawString - endPos;
		aString = std::string(startOfRawString,(endOfStrippedString - startOfStrippedString));
	}

	void stripLeadingAndTrailingSpaces(std::string& aString)
	{
		stripLeadingSpaces(aString);
		stripTrailingSpaces(aString);
	}

	std::string integerToString( unsigned long anInt)
	{
		// max unsigned == 4294967295, ergo 10 chars
		char intAsString[11];// = {0};
		sprintf(intAsString,"%lu",anInt);
		return std::string(intAsString);
	}

	std::string integerToString( long anInt)
	{
		// max unsigned == 4294967295, ergo 10 chars
		char intAsString[11];// = {0};
		sprintf(intAsString,"%ld",anInt);
		return std::string(intAsString);
	}

	unsigned long stringToUnsignedLong(const std::string& aString)  throw( std::invalid_argument)
	{
		// atol return 0L if the string cannot be converted to a number.
		// So check whether all chars in the string are digits. If not, throw an exception.
		for(unsigned long i = 0;i<aString.length();i++)
		{
			// isdigit returns 0 if the char is no digit
			if( isdigit(aString[i]) == 0)
			{
				// char string may contain a sign!!!
				// This conversion only allows it to be the very first char.
				// No blanks are allowed before the sign.
				if( ((aString[i] == '-') || (aString[i] == '+')) && i == 0) break;
				std::string excText = std::string("Cannot convert \"") + aString + std::string("\" to a unsigned long.");
				throw std::invalid_argument(excText);
			}
		}
		return atol(aString.c_str());
	}

	long stringToLong(const std::string& aString) throw( std::invalid_argument)
	{
		// atol return 0L if the string cannot be converted to a number.
		// So check whether all chars in the string are digits. If not, throw an exception.
		for(unsigned long i = 0;i<aString.length();i++)
		{
			// isdigit returns 0 if the char is no digit
			if( isdigit(aString[i]) == 0)
			{
				// char string may contain a sign!!!
				// This conversion only allows it to be the very first char.
				// No blanks are allowed before the sign.
				if( ((aString[i] == '-') || (aString[i] == '+')) && i == 0) break;
				std::string excText = std::string("Cannot convert \"") + aString + std::string("\" to a long.");
				throw std::invalid_argument(excText);
			}
		}
		return atol(aString.c_str());
	}

	std::string floatToString(float aFloat)
	{
		// atof handles max of 100 chars so I assume that there are no bigger numbers
		char floatAsString[101];
		sprintf(floatAsString,"%f",aFloat);
		return std::string(floatAsString);
	}

	float stringToFloat(const std::string& aString) throw( std::invalid_argument)
	{
		std::string workingString = aString;
		// atol return 0L if the string cannot be converted to a number.
		// So check whether all chars in the string are digits. If not, throw an exception.

		if( workingString[0] == '-'  || workingString[0] == '+' )
		{
			for(unsigned long i = 1;i<workingString.length();i++)
			{
				// isdigit returns 0 if the char is no digit
				if( isdigit(workingString[i]) == 0)
				{
					// char string may contain a sign.
					// This conversion only allows it to be the very first char.
					// No blanks are allowed before the sign.
					if(workingString[i] == '.')
					{
						break;
					}
					if(workingString[i] == ',')
					{
						workingString.replace(i,1,".");
						break;
					}
					std::string excText = std::string("Cannot convert \"") + workingString + std::string("\" to a float.");
					throw std::invalid_argument(excText);
				}
		 	}
		}else
		{
			for(unsigned long i = 0;i<workingString.length();i++)
			{
				// isdigit returns 0 if the char is no digit
				if( isdigit(workingString[i]) == 0)
				{
					// char string may contain a sign.
					// This conversion only allows it to be the very first char.
					// No blanks are allowed before the sign.
					if(workingString[i] == '.')
				{
					break;
				}
				if(workingString[i] == ',')
				{
					workingString.replace(i,1,".");
					break;
				}
				std::string excText = std::string("Cannot convert \"") + workingString + std::string("\" to a float.");
				throw std::invalid_argument(excText);
			}
		 }
	  }
	  return (float)atof(workingString.c_str());
	}

	std::string strprintf(const char* fmt, ...)
	{
		// The arguments can expand the buffer 1000 chars. Must be enought, like 640 kb.
		char* buffer = new char[strlen(fmt)+1000];
		va_list ap;
		va_start(ap, fmt);
		long result = vsprintf(buffer, fmt, ap);
		if(result<0) throw std::length_error("result<0");
		va_end(ap);
		std::string returnString(buffer);
		delete [] buffer;
		return returnString;
	}

	// These functions should be replaced by a parser class some day as they are tied to the ATextProfile class
	// The string will be split at the first occurence of aToken and the remainder of the string
	// will be put into aSecondString
	void splitString( const std::string& aStringToSplit, std::string& aFirstString, const char aToken, std::string& aSecondString)
	{
		const char* stringToSplit	 = aStringToSplit.data();
		size_t stringLength	 = aStringToSplit.length();
		unsigned long currentPosition = 0;

		// Find the first occurence of aToken
		while((*(stringToSplit+currentPosition) != aToken))
		{
			if(currentPosition == stringLength) // Token not found
			{
				aFirstString = aStringToSplit;
				aSecondString = "";
				return;
			}
			currentPosition++;
		}

		aFirstString   = std::string(stringToSplit,currentPosition);

		// Eat the token
		currentPosition++;

		aSecondString  = std::string(stringToSplit+currentPosition, stringLength-currentPosition);
	}

	void splitString( const std::string& aStringToSplit, std::string& aFirstString, const char aFirstToken, std::string& aSecondString, const char aSecondToken, std::string& aThirdString)
	{
		std::string remainder;
		splitString(aStringToSplit, aFirstString, aFirstToken, remainder);
		splitString(remainder, aSecondString, aSecondToken,aThirdString);
	}

	void splitString(const std::string& aStringToSplit, std::string& aFirstString, const char aFirstToken, std::string& aSecondString, const char aSecondToken, std::string& aThirdString, const char aThirdToken, std::string& aFourthString)
	{
		std::string toSplit = aStringToSplit;
		std::string remainder;

		splitString(toSplit, aFirstString, aFirstToken, remainder);

		toSplit = remainder;
		splitString(toSplit, aSecondString, aSecondToken, remainder);

		toSplit = remainder;
		splitString(toSplit, aThirdString, aThirdToken, aFourthString);
	}

	std::string bracketize(const std::string& aString)
	{
		std::string bracketizedString(aString);

		if (bracketizedString[0] != '[')
		{
			bracketizedString = std::string((char*)"[") + bracketizedString + std::string((char*)"]");
		}
		return bracketizedString;
	}

	std::string bracketize(const char* aString)
	{
		return bracketize(std::string(aString));
	}
} // namespace Danu
