#ifndef DanuString_hpp
#define DanuString_hpp

#include "DanuConfig.h"
#include "Exception.hpp"

#include <cstdio>
#include <string>

namespace Danu
{
	void DANUIMPEXP stripLeadingSpaces(std::string& aString);
	void DANUIMPEXP stripTrailingSpaces(  std::string& aString);
	void DANUIMPEXP stripLeadingAndTrailingSpaces(std::string& aString);

	std::string DANUIMPEXP integerToString( unsigned long anInt);
	std::string DANUIMPEXP integerToString( long anInt);

	// These conversion function throw an exception if aString cannot be converted
	// This is done because than they can be used in automatic conversion functions.
	// Wrong conversions must be debugged during development, not during releases
	unsigned long DANUIMPEXP stringToUnsignedLong(const std::string& aString) throw( std::invalid_argument);
	long DANUIMPEXP stringToLong(const std::string& aString) throw( std::invalid_argument);

	std::string DANUIMPEXP floatToString(float aFloat);
	float DANUIMPEXP stringToFloat(const std::string& aString)throw( std::invalid_argument);

	// Use this to create a string with printf like syntax
	std::string DANUIMPEXP strprintf(const char* fmt, ...);

	// These functions should be replaced by a parser class some day as they are tied to the ATextProfile class
	// If token occurs more than 1 time in the stringToSplit the string is split at the first occurrence.
	void DANUIMPEXP splitString(const std::string& aStringToSplit, std::string& aFirstString, const char aFirstToken, std::string& aSecondString);
	void DANUIMPEXP splitString(const std::string& aStringToSplit, std::string& aFirstString, const char aFirstToken, std::string& aSecondString, const char aSecondToken, std::string& aThirdString);
	void DANUIMPEXP splitString(const std::string& aStringToSplit, std::string& aFirstString, const char aFirstToken, std::string& aSecondString, const char aSecondToken, std::string& aThirdString, const char aThirdToken, std::string& aFourthString);

	std::string DANUIMPEXP bracketize(const std::string& aSectionName);
	std::string DANUIMPEXP bracketize(const char* aSectionName);
} // namespace Danu

#endif // DanuString_hpp

