#ifndef EventSemaphore_hpp
#define EventSemaphore_hpp

#include <pthread.h>
#include "AbstractWaitableObject.hpp"
#include "MutexSemaphore.hpp"

namespace Danu
{
	typedef  pthread_cond_t EventHandle;

	class DANUIMPEXP EventSemaphore : public AbstractWaitableObject
	{
		public:
			EventSemaphore();
			~EventSemaphore();

			const EventHandle& getHandle() const {return handle;}
			EventHandle& getHandle() {return handle;}

			/// \name AbstractLockableObject interface
			/// @{
			virtual void                              wait( long aTimeOut = -1);
			virtual void                              signal();
			virtual void                              broadcast();
			/// @}
		protected:
			/// \name Common (de)initialization functions
			/// @{
			virtual void init();
			virtual void deinit();
			/// @}
		private:
			EventHandle	handle;
			pthread_condattr_t eventAttribute;
			MutexSemaphore mutex;

	};
}; // namespace Danu

#endif // EventSemaphore_hpp
