#ifndef MutexSemaphore_hpp
#define MutexSemaphore_hpp

#include "DanuConfig.h"
#include "AbstractLockableObject.hpp"
#include <pthread.h>

namespace Danu
{
	typedef  pthread_mutex_t MutexHandle;

	/**
	For more info about the implementation of this class: it is implemented with POSIX semantics.
	No support for interprcess MutexSemaphore as the PThreads library does not support it (yet)
	*/
	class DANUIMPEXP MutexSemaphore : public AbstractLockableObject
	{
		public:
			MutexSemaphore();
			~MutexSemaphore();

			const MutexHandle&	getHandle() const {return handle;}
			MutexHandle&	getHandle() {return handle;}

			/// \name AbstractLockableObject interface
			/// @{
			virtual void                              lock( long aTimeOut = -1);
			virtual void                              trylock();
			virtual void                              unlock();
			/// @}

		protected:
			/// \name Common (de)initialization functions
			/// @{
			virtual void init();
			virtual void deinit();
			/// @}

		private:
			MutexHandle				handle;
			pthread_mutexattr_t		mutexAttribute;
	};
}; // namespace Danu
#endif // MutexSemaphore_hpp
