#ifndef Thread_hpp
#define Thread_hpp

#include "DanuConfig.h"
#include "ThreadFunction.hpp"
#include "MutexSemaphore.hpp"
#include <pthread.h>

namespace Danu
{
	// See pthreads.txt in the docs directory for instructions on installing pthread support for the compilers
	typedef  pthread_t ThreadHandle;
	typedef  unsigned long ThreadId;

	class CurrentThread;

	class DANUIMPEXP Thread
	{
		public:
			// A typedef for a C function that can be run by a thread with default C linkage
			typedef void* (* CFunctionPtr) (void*);

			Thread();
			Thread(	CFunctionPtr aCFunctionPtr,
					void* voidPointerFunctionArgument);
			Thread(	AbstractThreadFunction* anAbstractThreadFunction);

			ThreadId                           	getThreadId() const;
			static ThreadId                     getCurrentThreadId();

			const ThreadHandle&					getThreadHandle() const;
			ThreadHandle&						getThreadHandle();
			static ThreadHandle					getCurrentThreadHandle();

			static CurrentThread&               current();

			void								start(	CFunctionPtr aCFunctionPtr,
														void* voidPointerFunctionArgument);
			void								start(AbstractThreadFunction* anAbstractThreadFunction);

			virtual void						stop();
			virtual void						suspend();
			virtual void						resume();

			bool								isMainThread();

		private:
			/// Class to initialize the threads environment
			class ThreadIds;
			class ThreadInitializer
			{
				public:
					ThreadInitializer();
					~ThreadInitializer();
					MutexSemaphore threadIdsMutex;
					ThreadIds* threadIds;
			}; // class ExceptionInitializer
			static ThreadInitializer threadInitializer;

			mutable ThreadHandle				threadHandle;
			mutable ThreadId					threadId;
			unsigned long						exitCode;
	};

	class DANUIMPEXP CurrentThread : public Thread
	{
		public:
			virtual void						suspend();
			virtual void						sleep(unsigned long aNumberOfMilliSeconds);

			virtual void						waitFor(const Thread& aThread);
			virtual void						waitFor(const ThreadHandle& aThreadHandle);
	}; // End of class declaration of CurrentThread
}

#endif // Thread_hpp
