#ifndef Trace_hpp
#define Trace_hpp

#include "DanuConfig.h"
#include "DanuString.hpp"
#include "Exception.hpp"
#include "MutexSemaphore.hpp"

namespace Danu
{
	class ThreadIndentionLevels;

	class DANUIMPEXP Trace
	{
		public:

			class TraceFunction : public Exception::TraceFunction
			{
			};

			Trace(	const char* aText    = 0,
              		long aLineNumber     = 0,
              		bool withTimeStamp   = false);
			Trace(	const std::string& aText,
              		long aLineNumber     = 0,
              		bool withTimeStamp   = false);
			~Trace();

			static void setTraceFunction(TraceFunction* aTraceFunction = 0);
			static TraceFunction* getTraceFunction();

			static void	enableTrace();
			static void	disableTrace();
			static bool isTraceEnabled();

			static void enableLineNumber();
			static void disableLineNumber();
			static bool isLineNumberEnabled();

			static void enableThreadId();
			static void disableThreadId();
			static bool isThreadIdEnabled();

			static void enableExceptionTracing();
			static void disableExceptionTracing();
			static bool isExceptionTracingEnabled();

			static void trace(	const std::string& aText,
								bool withTimeStamp = Trace::isTraceEnabled());
			static void trace(	const char* aText,
								bool withTimeStamp = Trace::isTraceEnabled());

			//	The two functions above resolve to this function after formatting the text
			static void traceFormattedString(	const std::string& aText,
												char* aTraceMarker);	// 	This text will be prepended
																		// 	After the text a newline will be inserted
																		//	as approriate

		protected:
			static std::string timeStamp();
			void traceConstructor(long aLineNumber);

		private:
			class TraceInitializer
			{
				public:
					TraceInitializer();
					~TraceInitializer();
					MutexSemaphore traceFunctionMutex;
					ThreadIndentionLevels* threadIndentionLevels;
			}; // class TraceInitializer;
			static TraceInitializer traceInitializer;

			static TraceFunction* traceFunction;
			
			static bool traceOn;
     		static bool traceLineNumber;
			static bool traceThreadId;
			static bool traceInitialized;
			static bool	traceExceptions;
			
			std::string traceText;
			bool traceTimeStamp;
	}; // class Trace
} // namespace Danu

#ifdef DEVELOP
	#define FUNCTRACE_DEVELOP()                 Danu::Trace trc(__DANUFUNCTION__, __DANULINE__)
	#define TRACE_DEVELOP(text)                 Danu::Trace::trace(text)

	#define TSFUNCTRACE_DEVELOP()               Danu::Trace trc(__DANUFUNCTION__, __DANULINE__,true)
	#define TSTRACE_DEVELOP(text)               Danu::Trace::trace(text,true)
#elif RELEASE
	#define FUNCTRACE_DEVELOP()
	#define TRACE_DEVELOP(text)

	#define TSFUNCTRACE_DEVELOP()
	#define TSTRACE_DEVELOP(text)
#else
#endif

#endif // Trace_hpp
