#ifndef TraceThreadIndention_hpp
#define TraceThreadIndention_hpp

#include "DanuConfig.h"
#include "Thread.hpp"
#include <map>
#include <iostream>

#define INDENTION_LEVEL 3;
	
namespace Danu
{
	typedef long IndentionLevel;
	typedef long IndentionLevelAmount;
	
	class ThreadIndentionLevel
	{
		public:
			ThreadIndentionLevel()
				:	threadHandle(0),
					indentionLevel(0)
			{
				indentionLevel -= INDENTION_LEVEL;
			}
			
			ThreadIndentionLevel& operator++()    // == ++aThreadIndentionLevel, prefix
			{
				indentionLevel += INDENTION_LEVEL;
				return *this;
			}
			
			ThreadIndentionLevel operator++(int)  // == aThreadIndentionLevel++, postfix
			{
				ThreadIndentionLevel til = *this;
				++(*this);
				return til;
			}	
			
			ThreadIndentionLevel& operator--()    // == --aThreadIndentionLevel, prefix
			{
				indentionLevel -= INDENTION_LEVEL;
				return *this;
			}
			
			ThreadIndentionLevel operator--(int)  // == aThreadIndentionLevel++, postfix
			{
				ThreadIndentionLevel til = *this;
				--(*this);
				return til;
			}
			
			ThreadIndentionLevel operator+(IndentionLevelAmount anAmount)
			{
				ThreadIndentionLevel til(*this);
				til.indentionLevel += anAmount;
				return til;
			}
			
			ThreadIndentionLevel operator-(IndentionLevelAmount anAmount)
			{
				ThreadIndentionLevel til(*this);
				til.indentionLevel -= anAmount;
				return til;
			}
			
			ThreadIndentionLevel& operator+=(IndentionLevelAmount anAmount)
			{
				indentionLevel += anAmount;
				return *this;
			}
			
			ThreadIndentionLevel& operator-=(IndentionLevelAmount anAmount)
			{
				indentionLevel -= anAmount;
				return *this;
			}
	
			Danu::ThreadHandle  threadHandle;
			IndentionLevel   	indentionLevel;
	};
	
	class ThreadIndentionLevels : public std::map< Danu::ThreadHandle, Danu::ThreadIndentionLevel >
	{
		public:
			typedef std::map< Danu::ThreadHandle  , Danu::ThreadIndentionLevel >  Base;
			ThreadIndentionLevel& operator[](const Danu::ThreadHandle  & aThreadHandle)
			{
				// If there is no ThreadIndentionLevel for aThreadHandle
				// a ThreadIndentionLevel will be created with a default ctor
				ThreadIndentionLevel& til = ((Base*)this)->operator[](aThreadHandle);
				return til;
			}
	};
} // namespace Danu
#endif // TraceThreadIndention_hpp
